/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* 
 * Authors : 
 *  JP Rosevear <jpr@ximian.com>
 *  Rodrigo Moya <rodrigo@ximian.com>
 *  Anders Carlsson <andersca@imendio.com>
 *
 * Copyright 2003, Novell, Inc.
 * Copyright 2004, Imendio AB
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of version 2 of the GNU General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef E_GW_CONNECTION_H
#define E_GW_CONNECTION_H

#include <glib-object.h>
#include <libsoup/soup-message.h>

#include <utils/e2k-properties.h>

G_BEGIN_DECLS

#define OGO_TYPE_CONNECTION            (ogo_connection_get_type ())
#define OGO_CONNECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), OGO_TYPE_CONNECTION, OGoConnection))
#define OGO_CONNECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), OGO_TYPE_CONNECTION, OGoConnectionClass))
#define OGO_IS_CONNECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OGO_TYPE_CONNECTION))
#define OGO_IS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), OGO_TYPE_CONNECTION))

typedef struct _OGoConnection        OGoConnection;
typedef struct _OGoConnectionClass   OGoConnectionClass;
typedef struct _OGoConnectionPrivate OGoConnectionPrivate;
typedef struct _OGoItem              OGoItem;


typedef enum {
	OGO_CONNECTION_STATUS_OK,
	OGO_CONNECTION_STATUS_ACCESS_DENIED,
	OGO_CONNECTION_STATUS_NOT_FOUND,
	OGO_CONNECTION_STATUS_UNKNOWN
} OGoConnectionStatus;

struct _OGoConnection {
	GObject               parent;
	OGoConnectionPrivate *priv;
};

struct _OGoConnectionClass {
	GObjectClass parent_class;
};

struct _OGoItem {
	gchar *uid;
	gchar *uri;
	gchar *etag;
	
	gchar *contents;
};

typedef gboolean (*OGoConnectionTestCallback) (OGoConnection *conn,
					       const char    *test_name,
					       gpointer       user_data);


GType               ogo_connection_get_type    (void);
OGoConnection      *ogo_connection_new         (const char   *uri, 
						const char   *username, 
						const char   *password);

OGoConnectionStatus ogo_connection_delete_item (OGoConnection *conn,
						const gchar   *uri,
						const gchar   *etag);

OGoConnectionStatus ogo_connection_create_item (OGoConnection *conn,
						const gchar   *item_path,
						const gchar   *name,
						const gchar   *new_contents,
						const gchar   *mime_type,
						gchar        **new_uri);

OGoConnectionStatus ogo_connection_modify_item (OGoConnection *conn,
						const gchar   *uri,
						const gchar   *new_contents,
						const gchar   *etag,
						const gchar   *mime_type);

OGoConnectionStatus ogo_connection_fetch_item_absolute (OGoConnection *conn,
							const gchar   *absolute_path,
							gchar        **item_contents);

OGoConnectionStatus ogo_connection_list_items  (OGoConnection *conn,
						const gchar   *item_path,
						GList        **items);

OGoConnectionStatus ogo_connection_fetch_items (OGoConnection  *conn,
						const gchar    *folder_path,
						GList         **items);


OGoConnectionStatus ogo_connection_propfind    (OGoConnection  *conn,
						const gchar    *uri,
						const gchar   **props,
						gint            nprops,
						E2kProperties **out_props,
						gint           *nout_props);
					     
OGoConnectionStatus ogo_connection_proppatch   (OGoConnection  *conn,
						const char     *uri, 
						E2kProperties  *props,
						gboolean        create, 
						char          **repl_uid);

OGoConnectionStatus ogo_connection_proppatch_new (OGoConnection *conn,
						  const char    *folder_uri,
						  const char    *object_name,
						  OGoConnectionTestCallback test_callback,
						  gpointer       user_data,
						  E2kProperties *props,
						  char         **location, 
						  char         **repl_uid);
					     
OGoConnectionStatus ogo_connection_logout       (OGoConnection  *cnc);

OGoConnectionStatus ogo_try_authenticate        (const gchar    *uri, 
						 const gchar    *username,
						 const gchar    *password);

gchar              *ogo_connection_get_item_uri (OGoConnection  *conn,
						 const gchar    *item_path);

void                ogo_item_list_free          (GList *items);
     
G_END_DECLS

#endif
